{   Bouncing. Used to check units SDL and SDLGfx

    Copyright (C) 2006 by Eike Lange (eike.lange@uni-essen.de)
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, version 2.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
    
    As a special exception, if you incorporate even large parts of the
    code of this demo program into another program with substantially
    different functionality, this does not cause the other program to
    be covered by the GNU General Public License. This exception does
    not however invalidate any other reasons why it might be covered
    by the GNU General Public License.
}

{ compile this program the usual way:
  gpc --automake bouncing.pas -o bouncing
}

program Bouncing;

uses Sdl, GPC;

const
  Width = 800;
  Height = 600;
  rLen = 10;

type
  TPos = record
    x, y, ex, ey, v: Real
  end;

var
  Info: PSdlVideoInfo;
  Screen: PSdlSurface;
  Bpp: Integer;
  Event: TSdlEvent;
  Finished: Boolean Value False;
  Timer: PSdlTimerId;
  Rects: array [0..3] of TSdlRect;
  Pos: array [0..1] of TPos;

procedure MoveRect (var p: TPos; var r: TSdlRect);
const
  Left = rLen;
  Right = Width - rLen;
  Top = rLen;
  Bottom = Height - rLen;
var
  nx, ny: Real;
begin
  nx := p.x + p.v * p.ex;
  ny := p.y + p.v * p.ey;

  { left and right }
  if nx <= Left then
    begin
      p.ex := - p.ex;
      nx := Left + (Left - nx)
    end
  else if nx >= Right then
    begin
      p.ex := - p.ex;
      nx := Right - (nx - Right)
    end;

  { top and bottom }
  if ny <= Top then
    begin
      p.ey := - p.ey;
      ny := Top + (Top - ny)
    end
  else if ny >= Bottom then
    begin
      p.ey := - p.ey;
      ny := Bottom - (ny - Bottom)
    end;

  p.x := nx;
  p.y := ny;
  r.x :=  Round (nx) - rLen div 2;
  r.y :=  Round (ny) - rLen div 2;
  r.w := rLen;
  r.h := rLen

end;

function TimerFunction (Interval: Cardinal; Dummy: Pointer): Cardinal;
var
  UserEvent: TSdlUserEvent;
  Event: TSdlEvent;
begin
  with UserEvent do
    begin
      aType := SdlUserEvent;
      Code := 0;
      Data1 := nil;
      Data2 := nil
    end;
  Event.aType := SdlUserEvent;
  Event.User := UserEvent;
  SdlPushEvent (@Event);
  Discard (Dummy);
  TimerFunction := Interval
end;

begin
  if SdlInit (SdlInitVideo or SdlInitTimer) < 0 then
    begin
      WriteLn (StdErr, 'SdlInit failed: ', SdlGetError);
      Halt
    end;
  AtExit (SdlQuit);
  Info := SdlGetVideoInfo;
  if Info^.VFmt^.BitsPerPixel > 8 then
    Bpp := Info^.VFmt^.BitsPerPixel
  else
    Bpp := 16;
  Screen := SdlSetVideoMode (Width, Height, Bpp, SdlHWSURFACE or SdlFULLSCREEN);
  if Screen = nil then
    begin
      WriteLn (StdErr, 'SdlSetVideoMode failed: ', SdlGetError);
      Halt
    end;
  
  Timer := SdlAddTimer (10, @TimerFunction, nil);
  Rects[0].x := 100;
  Rects[0].y := 100;
  Rects[0].w := rLen;
  Rects[0].h := rLen;
  Rects[2] := Rects[0];
  
  Pos[0].x := 100.0;
  Pos[0].y := 100.0;
  Pos[0].ex := 0.707;
  Pos[0].ey := 0.707;
  Pos[0].v := 9.7;

  Pos[1] := Pos[0];
  Pos[1].ex := -0.707;
  Pos[1].v := 2;
  
  
  while SdlWaitEvent (Event) and (not Finished) do
    begin
        if (Event.aType = SdlQuitEvent) or (Event.aType = SdlKeyDown) then
          Finished := True
        else if Event.aType = SdlUserEvent then
          begin
            MoveRect (Pos[0], Rects[1]);
            MoveRect (Pos[1], Rects[3]);
            SdlFillRect (Screen, @Rects[0], SdlMapRGB (Screen^.Format, 0, 0, 0));
            SdlFillRect (Screen, @Rects[2], SdlMapRGB (Screen^.Format, 0, 0, 0));
            SdlFillRect (Screen, @Rects[1], SdlMapRGB (Screen^.Format, 255, 0, 0));
            SdlFillRect (Screen, @Rects[3], SdlMapRGB (Screen^.Format, 0, 255, 0));
            SdlUpdateRects (Screen, 4, PSdlRects (@Rects));
            Rects[0] := Rects[1];
            Rects[2] := Rects[3]
          end
    end
end.
