{   GNU Pascal Interface to SDLGfx. 
    Copyright (C) 2005 by Eike Lange (eike.lange@uni-essen.de)
    
    This unit is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This unit is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this unit; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Eike Lange, eike.lange@uni-essen.de
    http://www.gnu-pascal.de/~contrib/eike/

    This unit was implemented using version 2.0.13 of libSDL_gfx. 
    Color Values MUST be of 16#RRGGBBAA, AA* means Anti Aliased
    Some header files and/or routines were not supported:
      Missing:
        SDL_imageFilter.h (I have no experiences with this)
        SDL_rotozoom.h (types mssing. They are not referenced in funtions)
}

{$if __GPC_RELEASE__ < 20050331}
{$error This unit requires GPC release 20050331, or newer.}
{$endif}

unit SdlGfx;

interface

uses Sdl, GPC;

type
  { Just for polygon routines }
  TPolyArray = array [0.. High (ShortInt) div SizeOf (ShortInt)] of ShortInt;

  {@section types from SDL_framerate.h }
  PFpsManager = ^TFpsManager;
  TFpsManager = record
    Framecount: CCardinal;
    RateTicks: ShortReal;
    LastTicks, Rate: CCardinal
  end;

var
  {@section consts from SDL_framerate.h }
  FpsUpperLimit:  Integer; attribute (name = '_p_FPS_UPPER_LIMIT', const); external;
  FpsLowerLimit:  Integer; attribute (name = '_p_FPS_LOWER_LIMIT', const); external;
  FpsDefault:     Integer; attribute (name = '_p_FPS_DEFAULT', const); external;


  {@section consts from SDL_gfxPrimitives.h }
  SdlGfxPrimitivesMajor: Integer; attribute (name = '_p_SDL_GFXPRIMITIVES_MAJOR', const); external;
  SdlGfxPrimitivesMinor: Integer; attribute (name = '_p_SDL_GFXPRIMITIVES_MINOR', const); external;
  SdlGfxPrimitivesMicro: Integer; attribute (name = '_p_SDL_GFXPRIMITIVES_MICRO', const); external;


  {@section consts from SDL_gfxPrimitives.h }
  SmoothingOff:  Integer; attribute (name = '_p_SMOOTHING_OFF', const); external;
  SmoothingOn:   Integer; attribute (name = '_p_SMOOTHING_ON', const); external;


  {@section routines from SDL_framerate.h }
procedure SdlInitFramerate (Manager: PFpsManager); attribute (name = 'SDL_initFramerate'); external;
function SdlSetFramerate (Manager: PFpsManager; Rate: Integer): Integer; attribute (name = 'SDL_setFramerate'); external;
function SdlGetFramerate (Manager: PFpsManager): Integer; attribute (name = 'SDL_getFramerate'); external;
procedure SdlFramerateDelay (Manager: PFpsManager); attribute (name = 'SDL_framerateDelay'); external;


  {@section routines from SDL_gfxPrimitives.h }
{ Pixel }
function PixelColor (Dest: PSdlSurface; x, y: ShortInt; Color: Cardinal): Integer; attribute (name = 'pixelColor', ignorable); external;
function PixelRGBA (Dest: PSdlSurface; x, y: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'pixelRGBA', ignorable); external;

{ Horizontal Line }
function HLineColor (Dest: PSdlSurface; x1, x2, y: ShortInt; Color: Cardinal): Integer; attribute (name = 'hlineColor', ignorable); external;
function HLineRGBA (Dest: PSdlSurface; x1, x2, y: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'hlineRGBA', ignorable); external;

{ Vertical Line }
function VLineColor (Dest: PSdlSurface; x, y1, y2: ShortInt; Color: Cardinal): Integer; attribute (name = 'vlineColor', ignorable); external;
function VLineRGBA (Dest: PSdlSurface; x, y1, y2: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'vlineRGBA', ignorable); external;

{ Rectangle }
function RectangleColor (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; Color: Cardinal): Integer; attribute (name = 'rectangleColor', ignorable); external;
function RectangleRGBA (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'rectangleRGBA', ignorable); external;

{ Filled rectangle (Box) }
function BoxColor (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; Color: Cardinal): Integer; attribute (name = 'boxColor', ignorable); external;
function BoxRGBA (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'boxRGBA', ignorable); external;

{ Line }
function LineColor (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; Color: Cardinal): Integer; attribute (name = 'lineColor', ignorable); external;
function LineRGBA (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'lineRGBA', ignorable); external;

{ Anti Aliased Line }
function AALineColor (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; Color: Cardinal): Integer; attribute (name = 'aalineColor', ignorable); external;
function AALineRGBA (Dest: PSdlSurface; x1, y1, x2, y2: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'aalineRGBA', ignorable); external;

{ Circle }
function CircleColor (Dest: PSdlSurface; x, y, r: ShortInt; Color: Cardinal): Integer; attribute (name = 'circleColor', ignorable); external;
function CircleRGBA (Dest: PSdlSurface; x, y, rad: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'circleRGBA', ignorable); external;

{ Anti Aliased Circle }
function AACircleColor (Dest: PSdlSurface; x, y, r: ShortInt; Color: Cardinal): Integer; attribute (name = 'aacircleColor', ignorable); external;
function AACircleRGBA (Dest: PSdlSurface; x, y, rad: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'aacircleRGBA', ignorable); external;

{ Filled Circle }
function FilledCircleColor (Dest: PSdlSurface; x, y, r: ShortInt; Color: Cardinal): Integer; attribute (name = 'filledCircleColor', ignorable); external;
function FilledCircleRGBA (Dest: PSdlSurface; x, y, rad: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'filledCircleRGBA', ignorable); external;

{ Ellipse }
function EllipseColor (Dest: PSdlSurface; x, y, rx, ry: ShortInt; Color: Cardinal): Integer; attribute (name = 'ellipseColor', ignorable); external;
function EllipseRGBA (Dest: PSdlSurface; x, y, rx, ry: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'ellipseRGBA', ignorable); external;

{ Anti Aliased Ellipse }
function AAEllipseColor (Dest: PSdlSurface; x, y, rx, ry: ShortInt; Color: Cardinal): Integer; attribute (name = 'aaellipseColor', ignorable); external;
function AAEllipseRGBA (Dest: PSdlSurface; x, y, rx, ry: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'aaellipseRGBA', ignorable); external;

{ Filled Ellipse }
function FilledEllipseColor (Dest: PSdlSurface; x, y, rx, ry: ShortInt; Color: Cardinal): Integer; attribute (name = 'filledEllipseColor', ignorable); external;
function FilledEllipseRGBA (Dest: PSdlSurface; x, y, rx, ry: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'filledEllipseRGBA', ignorable); external;

{ Filled Pie }
function FilledPieColor (Dest: PSdlSurface; x, y, rad, AngleBegin, AngleEnd: ShortInt; Color: Cardinal): Integer; attribute (name = 'filledpieColor', ignorable); external;
function FilledPieRGBA (Dest: PSdlSurface; x, y, rad, AngleBegin, AngleEnd: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'filledpieRGBA', ignorable); external;

{ Trigon }
function TrigonColor (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; Color: Cardinal): Integer; attribute (name = 'trigonColor', ignorable); external;
function TrigonRGBA (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'trigonRGBA', ignorable); external;

{ AA-Trigon }
function AATrigonColor (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; Color: Cardinal): Integer; attribute (name = 'aatrigonColor', ignorable); external;
function AATrigonRGBA (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'aatrigonRGBA', ignorable); external;

{ Filled Trigon }
function FilledTrigonColor (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; Color: Cardinal): Integer; attribute (name = 'filledTrigonColor', ignorable); external;
function FilledTrigonRGBA (Dest: PSdlSurface; x1, y1, x2, y2, x3, y3: ShortInt; r, g, b, a: ByteCard): Integer; attribute (name = 'filledTrigonRGBA', ignorable); external;

{ Polygon }
function PolygonColor (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; Color: Cardinal): Integer; attribute (name = 'polygonColor', ignorable); external;
function PolygonRGBA (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; r, g, b, a: ByteCard): Integer; attribute (name = 'polygonRGBA', ignorable); external;
  
{ AA-Polygon }
function AAPolygonColor (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; Color: Cardinal): Integer; attribute (name = 'aapolygonColor', ignorable); external;
function AAPolygonRGBA (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; r, g, b, a: ByteCard): Integer; attribute (name = 'polygonRGBA', ignorable); external;

{ Filled Polygon }
function FilledPolygonColor (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; Color: Cardinal): Integer; attribute (name = 'filledPolygonColor', ignorable); external;
function FilledPolygonRGBA (Dest: PSdlSurface; var vx, vy: TPolyArray; Num: Integer; r, g, b, a: ByteCard): Integer; attribute (name = 'filledPolygonRGBA', ignorable); external;

{ Bezier }
function BezierColor (Dest: PSdlSurface; var vx, vy: TPolyArray; NumPoints, NumSteps: Integer; Color: Cardinal): Integer; attribute (name = 'bezierColor', ignorable); external;
function BezierRGBA (Dest: PSdlSurface; var vx, vy: TPolyArray; NumPoints, NumSteps: Integer; r, g, b, a: ByteCard): Integer; attribute (name = 'bezierRGBA', ignorable); external;

{ 8x8 Characters/Strings }
function CharacterColor (Dest: PSdlSurface; x, y: ShortInt; c: Char; Color: Cardinal): Integer; attribute (name = 'characterColor', ignorable); external;
function CharacterRGBA (Dest: PSdlSurface; x, y: ShortInt; c: Char; r, g, b, a: ByteCard): Integer; attribute (name = 'characterRGBA', ignorable); external;
function StringColor (Dest: PSdlSurface; x, y: ShortInt; s: TString; Color: Cardinal): Integer; attribute (ignorable);
function StringRGBA (Dest: PSdlSurface; x, y: ShortInt; s: TString; r, g, b, a: ByteCard): Integer; attribute (ignorable);
procedure GfxPrimitivesSetFont (FontData: Pointer; Cwf, Chf: Integer); attribute (name = 'gfxPrimitivesSetFont'); external;


  {@section routines from SDL_rotozoom.h }
{ Rotozoom }
function RotozoomSurface (Src: PSdlSurface; Angle, Zoom: Real; Smooth: Integer): PSdlSurface; attribute (name = 'rotozoomSurface'); external;
function RotozoomSurfaceXY (Src: PSdlSurface; Angle, ZoomY, ZoomX: Real; Smooth: Integer): PSdlSurface; attribute (name = 'rotozoomSurfaceXY'); external;
procedure RotozoomSurfaceSize (Width, Height: Integer; Angle, Zoom: Real; var DestWidth, DestHeight: Integer); attribute (name = 'rotozoomSurfaceSize'); external;
procedure RotozoomSurfaceSizeXY (Width, Height: Integer; Angle, ZoomX, ZoomY: Real; var DestWidth, DestHeight: Integer); attribute (name = 'rotozoomSurfaceSizeXY'); external;

{ ZoomSurface }
function ZoomSurface (Src: PSdlSurface; ZoomX, ZoomY: Real; Smooth: Integer): PSdlSurface; attribute (name = 'zoomSurface'); external;
procedure ZoomSurfaceSize (Width, Height: Integer; ZoomX, ZoomY: Real; var DestWidth, DestHeight: Integer); attribute (name = 'zoomSurfaceSize'); external;



implementation

{$L sdlgfxc.c}
{$L SDL_gfx}


function CStringColor (Dest: PSdlSurface; x, y: ShortInt; s: PChar;
  Color: Cardinal): Integer; attribute (name = 'stringColor'); external;
function StringColor (Dest: PSdlSurface; x, y: ShortInt; s: TString;
  Color: Cardinal): Integer;
begin
  StringColor := CStringColor (Dest, x, y, String2CString (s), Color)
end;

function CStringRGBA (Dest: PSdlSurface; x, y: ShortInt; s: PChar;
  r, g, b, a: ByteCard): Integer; attribute (name = 'stringRGBA'); external;
function StringRGBA (Dest: PSdlSurface; x, y: ShortInt; s: TString;
  r, g, b, a: ByteCard): Integer; 
begin
  StringRGBA := CStringRGBA (Dest, x, y, String2CString (s), r, g, b, a)
end;

end.
